@@ -152,6 +152,17 @@ static int parse_ftm_target(struct nl_msg *msg, char *str, int peer_index)
 			NLA_PUT_U32(msg, NL80211_PMSR_FTM_REQ_ATTR_PREAMBLE,
 				    NL80211_PREAMBLE_HE);
 			preamble = true;
+		} else if (strncmp(pos, "lmr_feedback", 12) == 0) {
+			NLA_PUT_FLAG(msg,
+				     NL80211_PMSR_FTM_REQ_ATTR_LMR_FEEDBACK);
+		} else if (strncmp(pos, "bss_color=", 10) == 0) {
+			NLA_PUT_U8(msg,
+				   NL80211_PMSR_FTM_REQ_ATTR_BSS_COLOR,
+				   strtol(pos + 10, &tmp, 0));
+			if (*tmp) {
+				printf("Invalid bss_color value!\n");
+				return HANDLER_RET_USAGE;
+			}
 		} else {
 			printf("Unknown parameter %s\n", pos);
 			return HANDLER_RET_USAGE;
@@ -282,6 +293,9 @@ static int handle_ftm_req_send(struct nl80211_state *state, struct nl_msg *msg,
 	const char *file;
 	int err;
 
+	if (argc < 1)
+		return HANDLER_RET_USAGE;
+
 	file = argv[0];
 	argc--;
 	argv++;
